@@ -1,5 +1,6 @@
 local bitshim = require("bitshim")
 local luautf8 = require("lua-utf8")
+local semver = require("semver")
 
 local utilities = {}
 
@@ -24,14 +25,16 @@ utilities.boolean = function (value, default)
   if value == "true" then return true end
   if value == "no" then preferbool(); return false end
   if value == "yes" then preferbool(); return true end
+  if value == nil then return default end
+  SU.error("Expecting a boolean value but got '" .. value .. "'")
   return default
 end
 
 local _skip_traceback_levels = 2
 
-utilities.error = function(message, bug)
+utilities.error = function (message, isbug)
   _skip_traceback_levels = 3
-  utilities.warn(message, bug)
+  utilities.warn(message, isbug)
   _skip_traceback_levels = 2
   io.stderr:flush()
   SILE.outputter:finish() -- Only really useful from the REPL but no harm in trying
@@ -39,9 +42,10 @@ utilities.error = function(message, bug)
   error(message, 2)
 end
 
-utilities.warn = function(message, bug)
+utilities.warn = function (message, isbug)
+  if SILE.quiet then return end
   io.stderr:write("\n! " .. message)
-  if SILE.traceback or bug then
+  if SILE.traceback or isbug then
     io.stderr:write(" at:\n" .. SILE.traceStack:locationTrace())
     if _skip_traceback_levels == 2 then
       io.stderr:write(debug.traceback("", _skip_traceback_levels) or "\t! debug.traceback() did not identify code location")
@@ -52,6 +56,11 @@ utilities.warn = function(message, bug)
   io.stderr:write("\n")
 end
 
+utilities.msg = function (message)
+  if SILE.quiet then return end
+  io.stderr:write("\n! " .. message .. "\n")
+end
+
 utilities.debugging = function (category)
   return SILE.debugFlags.all and category ~= "profile" or SILE.debugFlags[category]
 end
@@ -86,23 +95,25 @@ utilities.gtoke = function (string, pattern)
 end
 
 utilities.deprecated = function (old, new, warnat, errorat, extra)
+  warnat, errorat = semver(warnat or 0), semver(errorat or 0)
+  local current = SILE.version and semver(SILE.version:match("v([0-9]*.[0-9]*.[0-9]*)")) or warnat
   -- SILE.version is defined *after* most of SILE loads. It’s available at
   -- runtime but not useful if we encounter deprecated code in core code. Users
   -- will never encounter this failure, but as a developer it’s hard to test a
   -- deprecation when core code refactoring is an all-or-nothing proposition.
-  -- Hence we fake it ‘till we make it, all deprecations internally are warings.
+  -- Hence we fake it ‘till we make it, all deprecations internally are warnings.
   local brackets = old:sub(1,1) == '\\' and "" or "()"
-  local _semver = SILE.version and SILE.version:match("v([0-9]*.[0-9]*.[0-9]*)") or warnat
   local _new = new and "Please use " .. (new .. brackets) .. " instead." or "Plase don't use it."
-  local msg = (old .. brackets) .. " was deprecated in SILE v" .. warnat .. ". " .. _new ..  (extra and "\n" .. extra .. "\n\n" or "")
-  if errorat and _semver >= errorat then
+  local msg = (old .. brackets) .. " was deprecated in SILE v" .. tostring(warnat) .. ". " .. _new ..  (extra and "\n" .. extra .. "\n\n" or "")
+  if errorat and current >= errorat then
     SU.error(msg)
-  elseif warnat and _semver >= warnat then
+  elseif warnat and current >= warnat then
     SU.warn(msg)
   end
 end
 
 utilities.debug = function (category, ...)
+  if SILE.quiet then return end
   if utilities.debugging(category) then
     local inputs = table.pack(...)
     for i, input in ipairs(inputs) do
@@ -299,11 +310,6 @@ utilities.cast = function (wantedType, value)
   wantedType = string.lower(wantedType)
   if wantedType:match(actualType)     then return value
   elseif actualType == "nil" and wantedType:match("nil") then return nil
-  elseif wantedType:match("integer") or wantedType:match("number") then
-    if type(value) == "table" and type(value.tonumber) == "function" then
-      return value:tonumber()
-    end
-    return tonumber(value)
   elseif wantedType:match("length")      then return SILE.length(value)
   elseif wantedType:match("measurement") then return SILE.measurement(value)
   elseif wantedType:match("vglue")       then return SILE.nodefactory.vglue(value)
@@ -312,6 +318,27 @@ utilities.cast = function (wantedType, value)
   elseif actualType == "nil" then SU.error("Cannot cast nil to " .. wantedType)
   elseif wantedType:match("boolean")     then return SU.boolean(value)
   elseif wantedType:match("string")      then return tostring(value)
+  elseif wantedType:match("number") then
+    if type(value) == "table" and type(value.tonumber) == "function" then
+      return value:tonumber()
+    end
+    local num = tonumber(value)
+    if not num then SU.error("Cannot cast '" .. value .. "'' to " .. wantedType) end
+    return num
+  elseif wantedType:match("integer") then
+    local num
+    if type(value) == "table" and type(value.tonumber) == "function" then
+      num = value:tonumber()
+    else
+      num = tonumber(value)
+    end
+    if not num then SU.error("Cannot cast '" .. value .. "'' to " .. wantedType) end
+    if not wantedType:match("number") and num % 1 ~= 0 then
+      -- Could be an error but since it wasn't checked before, let's just warn:
+      -- Some packages might have wrongly typed settings, for instance.
+      SU.warn("Casting an integer but got a float number " .. num)
+    end
+    return num
   else SU.error("Cannot cast to unrecognized type " .. wantedType)
   end
 end
@@ -363,6 +390,29 @@ utilities.walkContent = function (content, action)
   end
 end
 
+--- Strip position, line and column recursively from a content tree.
+-- This can be used to remove position details where we do not want them,
+-- e.g. in table of contents entries (referring to the original content,
+-- regardless where it was exactly, for the purpose of checking whether
+-- the table of contents changed.)
+--
+utilities.stripContentPos = function (content)
+  if type(content) ~= "table" then
+    return content
+  end
+  local stripped = {}
+  for k, v in pairs(content) do
+    if type(v) == "table" then
+      v = SU.stripContentPos(v)
+    end
+    stripped[k] = v
+  end
+  if content.id or content.command then
+    stripped.pos, stripped.col, stripped.lno = nil, nil, nil
+  end
+  return stripped
+end
+
 utilities.rateBadness = function(inf_bad, shortfall, spring)
   if spring == 0 then return inf_bad end
   local bad = math.floor(100 * math.abs(shortfall / spring) ^ 3)
